/*
 * AbiCollab - Code to enable the modification of remote documents.
 * Copyright (C) 2005 by Martin Sevior
 * Copyright (C) 2006 by Marc Maurer <uwog@uwog.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef ABI_COLLAB_EXPORT_H
#define ABI_COLLAB_EXPORT_H

#include "ut_types.h"
#include "pt_Types.h"
#include "px_ChangeRecord.h"
#include "ut_stack.h"
#include "stdio.h"
#include "xav_Listener.h"
#include "pl_Listener.h"
#include "ut_string_class.h"

class FL_DocLayout;
class PD_Document;
class UT_Stack;
class PX_ChangeRecord;
class ChangeAdjust;
class AbiCollab;

class ABI_EXPORT ABI_Collab_Export : public PL_DocChangeListener
{

friend class AbiCollab;

public:
	ABI_Collab_Export(AbiCollab* pAbiCollab, PD_Document* pDoc);
	virtual ~ABI_Collab_Export();

	virtual bool		populate(PL_StruxFmtHandle sfh,
								 const PX_ChangeRecord* pcr) { return true; }

	virtual bool		populateStrux(PL_StruxDocHandle sdh,
									  const PX_ChangeRecord* pcr,
									  PL_StruxFmtHandle* psfh) { return true; }

	virtual bool		change(PL_StruxFmtHandle sfh,
							   const PX_ChangeRecord* pcr);

	virtual void		deferNotifications(void) {}
	virtual void		processDeferredNotifications(void) {}

	virtual bool		insertStrux(PL_StruxFmtHandle sfh,
									const PX_ChangeRecord* pcr,
									PL_StruxDocHandle sdh,
									PL_ListenerId lid,
									void (*pfnBindHandles)(PL_StruxDocHandle sdhNew,
															PL_ListenerId lid,
															PL_StruxFmtHandle sfhNew));

	virtual bool		signal(UT_uint32 iSignal);

	virtual	PLListenerType getType() const { return PTL_CollabExport; }

	virtual void            setNewDocument(PD_Document * pDoc);
	virtual void            removeDocument(void);

	bool                    isGlobEnd(UT_Byte istart, UT_Byte istop);

	const UT_GenericVector<ChangeAdjust *> * getAdjusts(void) const 
		{ return & m_vecAdjusts;}
	UT_GenericVector<ChangeAdjust *> * getAdjusts(void)
		{ return & m_vecAdjusts;}
	void addFakeImportAdjust(const UT_UTF8String& docUUID, UT_sint32 iRev); // used to initialize the adjustment stack when a remote document is received
	
private:

	bool 				buildPacket(bool bPopulate, const PX_ChangeRecord* pcr,
									UT_UTF8String& sPacket,
									UT_sint32& iAdjust,
									UT_sint32& iLength);

	bool				mapAttributesToString(UT_sint32 indx, UT_UTF8String& sAttribs);

	bool				mapChangeRecordToString(const PX_ChangeRecord* pcr,
									UT_UTF8String& sType, UT_UTF8String& sArgs,
									UT_sint32 & iAdjust, UT_sint32 & iLength);

	//! Document which is client of this DocListener
	PD_Document*		m_pDoc;

	UT_Stack			m_sLastContainerLayout;

	AV_ChangeMask		m_chgMaskCached;
	bool				m_bCacheChanges;

	UT_sint32			m_iSpanIndex;
	UT_sint32			m_iBlockIndex;
	UT_sint32			m_iSectionIndex;

	AbiCollab *			m_pAbiCollab;
	UT_GenericVector<ChangeAdjust *>        m_vecAdjusts;
	UT_UTF8String		m_sPacket;

	bool				m_bDoingGlob;
	UT_sint32			m_iGlobPos;
	UT_sint32			m_iGlobLength;
	UT_sint32			m_iGlobAdjust;
	UT_sint32			m_iGlobRev;
	UT_sint32			m_iGlobRemoteRev;
	UT_Byte				m_iGlobFlag;
};

#endif /* ABI_COLLAB_EXPORT_H */
